@@ -4,46 +4,41 @@ data_location = os.path.join(__dir__, "data")
 src = "https://github.com/picolibc/picolibc"
 
 # Module version
-version_str = "1.7.4.post149"
-version_tuple = (1, 7, 4, 149)
+version_str = "1.7.5.post126"
+version_tuple = (1, 7, 5, 126)
 try:
     from packaging.version import Version as V
-    pversion = V("1.7.4.post149")
+    pversion = V("1.7.5.post126")
 except ImportError:
     pass
 
 # Data version info
-data_version_str = "1.7.4.post27"
-data_version_tuple = (1, 7, 4, 27)
+data_version_str = "1.7.5.post0"
+data_version_tuple = (1, 7, 5, 0)
 try:
     from packaging.version import Version as V
-    pdata_version = V("1.7.4.post27")
+    pdata_version = V("1.7.5.post0")
 except ImportError:
     pass
-data_git_hash = "29d7282a37fca022c8ea3214f53f0dbcc5173b63"
-data_git_describe = "1.7.4-27-g29d7282a3"
+data_git_hash = "f3981f6c16a7b690f1697c9a7f986964bc2e386f"
+data_git_describe = "1.7.5-0-gf3981f6c1"
 data_git_msg = """\
-commit 29d7282a37fca022c8ea3214f53f0dbcc5173b63
+commit f3981f6c16a7b690f1697c9a7f986964bc2e386f
 Author: Keith Packard <keithp@keithp.com>
-Date:   Mon Jan 3 11:03:12 2022 -0800
+Date:   Mon Mar 7 22:59:36 2022 -0800
 
-    Add double-underscore PICOLIBC version macros
-    
-    When I initially created the meson build files, I mistakenly used only
-    a single leading underscore in all of the version macros. This adds
-    double leading underscore names, leaving the single underscore names
-    for anyone using them.
+    Version 1.7.5
     
     Signed-off-by: Keith Packard <keithp@keithp.com>
 
 """
 
 # Tool version info
-tool_version_str = "0.0.post122"
-tool_version_tuple = (0, 0, 122)
+tool_version_str = "0.0.post126"
+tool_version_tuple = (0, 0, 126)
 try:
     from packaging.version import Version as V
-    ptool_version = V("0.0.post122")
+    ptool_version = V("0.0.post126")
 except ImportError:
     pass
 
